# Generated by roxygen2: do not edit by hand

S3method(as.phylo,slendr_phylo)
S3method(print,slendr_map)
S3method(print,slendr_model)
S3method(print,slendr_pop)
S3method(print,slendr_region)
S3method(print,slendr_ts)
S3method(summary,slendr_nodes)
export("%>%")
export(animate_model)
export(area)
export(as.phylo.slendr_phylo)
export(check_dependencies)
export(check_env)
export(clear_env)
export(compile_model)
export(distance)
export(expand_range)
export(explore_model)
export(extract_parameters)
export(gene_flow)
export(get_python)
export(init_env)
export(join)
export(move)
export(msprime)
export(overlap)
export(plot_map)
export(plot_model)
export(population)
export(print.slendr_ts)
export(read_model)
export(region)
export(reproject)
export(resize)
export(schedule_sampling)
export(set_dispersal)
export(set_range)
export(setup_env)
export(shrink_range)
export(slim)
export(substitute_values)
export(subtract)
export(ts_afs)
export(ts_ancestors)
export(ts_coalesced)
export(ts_descendants)
export(ts_divergence)
export(ts_diversity)
export(ts_draw)
export(ts_edges)
export(ts_eigenstrat)
export(ts_extend)
export(ts_f2)
export(ts_f3)
export(ts_f4)
export(ts_f4ratio)
export(ts_fst)
export(ts_genotypes)
export(ts_ibd)
export(ts_load)
export(ts_metadata)
export(ts_mutate)
export(ts_names)
export(ts_nodes)
export(ts_phylo)
export(ts_read)
export(ts_recapitate)
export(ts_samples)
export(ts_save)
export(ts_segregating)
export(ts_simplify)
export(ts_table)
export(ts_tajima)
export(ts_tracts)
export(ts_tree)
export(ts_vcf)
export(ts_write)
export(world)
importFrom(ape,as.phylo)
importFrom(ggplot2,aes)
importFrom(ggplot2,arrow)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,geom_sf_label)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,unit)
importFrom(magrittr,"%>%")
